
from unittest import TestCase

import io
import json
import yaml
import xml.etree.cElementTree as xmlet
import os
import itertools
from dax import bin
from dax import dax_tools_utils


from dax import XnatUtils
from dax import assessor_utils
from dax.tests import unit_test_common_processor_yamls as yamls
from dax.tests.common_session_tools import SessionTools
from dax.yaml_doc import YamlDoc
from dax import AutoProcessor


# TODO: BenM/xnat refactor/ these sanity checks should be refactored into
# integration tests and performed on a test database that is explicitly
# setup / torn down within the scope of a test suite execution.
# Remove everything that doesn't make it into an integration test _before_
# the pull request back to vuiis/dax!

host165 = 'http://10.1.1.165'
host17 = 'http://10.1.1.17'
host = host17
proj_id = 'proj1'
subj_id = 'subj1'
sess_id = 'sess1'
scan_id = '1'
assr_id = 'asr1'

asrxsitype = 'proc:genProcData'
scanxsitype = 'xnat:mrScanData'

expstr = '/data/projects/{}/subjects/{}/experiments/{}'.format(
    proj_id, subj_id, sess_id)
scanstr = '/scans/{}'


# test dimensions:
#
# usable - processor:
# not set, false, true
#
# usable - scan / assessor:
# (unusable, questionable, usable, preferred), various
#
# types - scan / assessor:
# (1 type, several types), 1 type?
#
# needs qc - processor
# not set, false, true
#
# required - resource
# not set, false, true
#
#
# qc status - assessor
# JOB_PENDING, NEEDS_QA, GOOD, PASSED_QA, FAILED, BAD, POOR, RERUN, REPROC
# DONOTRUN, FAILED_NEEDS_REPROC, PASSED_EDITED_QA,
# OPEN_QA_LIST(RERUN, REPROC), BAD_QA_STATUS(FAILED, BAD, POOR, DONOTRUN)
#
# job status - assessor
# NO_DATA, NEED_TO_RUN, NEED_INPUTS, JOB_RUNNING, JOB_FAILED,
# READY_TO_UPLOAD, UPLOADING, COMPLETE, READY_TO_COMPLETE, DOES_NOT_EXIST,
# OPEN_STATUS_LIST(NEED_TO_RUN, UPLOADING, JOB_RUNNING, READY_TO_COMPLETE,
# JOB_FAILED), JOB_BUILT
#
# inputs
# no scans, 1 scan, n scans
# no assessors, 1 assessor, n assessors
# zero variables, one variable, n variables (from an input)
#
# select
# not set, foreach, foreach(other), one, some, all

t1_params = {
    'xsitype': scanxsitype,
    'type': "T1",
    'quality': "usable",
    'files': [
        ('NIFTI', ['images.nii']),
        ('SNAPSHOTS', ['snapshot.gif', 'snapshot(1).gif'])
    ]
}

t2_params = {
    'xsitype': scanxsitype,
    'type': "T2",
    'quality': "usable",
    'files': [
        ('NIFTI', ['images.nii']),
        ('SNAPSHOTS', ['snapshot.gif', 'snapshot(1).gif'])
    ]
}

flair_params = {
    'xsitype': scanxsitype,
    'type': "FLAIR",
    'quality': "usable",
    'files': [
        ('NIFTI', ['images.nii']),
        ('SNAPSHOTS', ['snapshot.gif', 'snapshot(1).gif'])
    ]
}

scan_presets = {
    't1': t1_params,
    't2': t2_params,
    'flair': flair_params
}


proc_x_params = {
    'xsitype': asrxsitype,
    'proctype': 'Proc_X_v1',
    'files': [
        ('SEG', ['seg.gz']),
    ]
}

assessor_presets = {
    'Proc_X_v1': proc_x_params
}


class ComponentTestBuild(TestCase):

    scan1_yaml = yamls.generate_yaml(
        scans=[
            {
                'name': 'scan1', 'types': 'T1', 'select': None, 'qc': None,
                'resources': [
                    {'type': 'NIFTI', 'name': 't1', 'required': None}
                ]
            }
        ]
    )

    @staticmethod
    def __generate_yamls(entries):
        yamldocs = []
        for e in entries:
            print(e)
            yamldocs.append(
                yamls.generate_yaml(
                    scans=[{
                        'name': 'scan1',
                        'types': 'T1',
                        'select': e['select'],
                        'qc': e['quality'],
                        'resources': [{
                            'type': 'NIFTI',
                            'name': 't1',
                            'required': e['required']
                        }]
                    }]
                )
            )

    @staticmethod
    def _setup_scans(session, scan_descriptors):
        for s, d in scan_descriptors:
            print((s, d))
            SessionTools.add_scan(session, s, scan_presets[d])

    @staticmethod
    def _setup_assessors(proj_id, subj_id, session):
        for i in range(1, 3):
            SessionTools.add_assessor(
                session, '{}-x-{}-x-{}-x-{}-x-Proc_X_v1'.format(
                    proj_id, subj_id, session.label(), i),
                assessor_presets['Proc_X_v1'],
                'no_inputs')

    def test_setup_old_assessors(self):
        intf = XnatUtils.get_interface(host)
        proj_id = 'proj1'
        subj_id = 'subj1'
        sess_ids = ['sess2', 'sess1']

        project = intf.select_project(proj_id)
        if not project.exists():
            self.assertTrue(False, 'proj1 should be pre-created for this test')

        subject = intf.select_subject(proj_id, subj_id)
        if not subject.exists():
            self.assertTrue(False, 'subj1 should be pre-created for this test')

        for s in sess_ids:
            session = intf.select_experiment(proj_id, subj_id, s)
            if not session.exists():
                self.assertTrue(
                    False, 'sess1 should be pre-created for this test')

            # delete and recreate scans
            scan_descriptors = [('1', 't1'), ('2', 't1'), ('11', 'flair')]
            ComponentTestBuild._setup_scans(session, scan_descriptors)

            # delete and recreate old assessors
            ComponentTestBuild._setup_assessors(proj_id, subj_id, session)

    def test_setup_session(self):
        proj_id = 'proj1'
        subj_id = 'subj1'
        sess_ids = ['sess1', 'sess2']
        intf = XnatUtils.get_interface(host=host)
        scan_descriptors = [
            ('1', 't1'), ('2', 't1'),
            ('11', 'flair'), ('12', 'flair'),
            ('21', 't2')]
        for sess_id in sess_ids:
            session = intf.select_experiment(proj_id, subj_id, sess_id)
            if not session.exists():
                self.assertTrue(False, "no such session")

            ComponentTestBuild._setup_scans(session, scan_descriptors)

    def test_clean_scans_from_test_session(self):
        proj_id = 'proj1'
        subj_id = 'subj1'
        sess_ids = ['sess1', 'sess2']
        # subj_id = 'subj2'
        # sess_ids = ['sess3', 'sess4']
        intf = XnatUtils.get_interface(host=host)
        for sess_id in sess_ids:
            session = intf.select_experiment(proj_id, subj_id, sess_id)
            if not session.exists():
                self.assertTrue(False, "no such session")

            for scn in session.scans():
                scn.delete()

    def test_clean_assessors_from_test_session(self):
        proj_id = 'proj1'
        subj_id = 'subj1'
        sess_ids = ['sess1', 'sess2']
        # subj_id = 'subj2'
        # sess_ids = ['sess3', 'sess4']
        intf = XnatUtils.get_interface(host=host)
        for sess_id in sess_ids:
            session = intf.select_experiment(proj_id, subj_id, sess_id)
            if not session.exists():
                self.assertTrue(False, "no such session")

            for asr in session.assessors():
                print((assessor_utils.full_label_from_assessor(asr)))
                asr.delete()

    # TODO: the text matrix that we run at the component level should be only
    # a small subset of the unit test processor parser test matrix
    def test_build_matrix(self):

        input_headers = ['xsitype', 'quality', 'type', 'files']
        input_xsitype = ['xnat:mrScanData']
        input_quality = ['unusable', 'usable', 'preferred']
        input_type = ['T1', 'T2']
        input_files = [
            [],
            [('NIFTI', ['images.nii'])],
            # [('NIFTI', ['images.nii']), ('SNAPSHOTS', ['snapshot.jpg.gz',
            # 'snapshot(1).jpg.gz'])]
        ]
        input_values = [input_xsitype, input_quality, input_type, input_files]
        input_table_values = itertools.product(*input_values)
        input_table = [[dict(zip(input_headers, r))] for r in input_table_values]

        headers = ['required', 'select', 'quality', 'inputs']
        required = [None, False, True]
        select = [None, 'foreach']
        quality = [None, False, True]
        input_fields = [[]] + [i for i in input_table]
        values = [required, select, quality, input_fields]
        table_values = itertools.product(*values)
        table = [dict(zip(headers, r)) for r in table_values]

        # generate the scan artefacts against which this is run
        for scans in input_table:
            for scan in scans:
                SessionTools.add_scan(None, 'scan1', scan)

        print((len(table)))

    def test_end_to_end_test(self):
        print((os.getcwd()))
        settings_location = os.getcwd()+'/settings/'
        processor_location = os.getcwd()+'/processors/'

        self.test_clean_assessors_from_test_session()

        self.test_setup_old_assessors()

        bin.build(
            settings_location + 'test.yaml',
            None,
            True,
            projects='proj1',
            sessions='sess1,sess2',
            mod_delta=None,
            proj_lastrun=None)

        bin.launch_jobs(
            settings_location + 'test.yaml',
            None,
            True,
            projects='proj1',
            sessions='sess1,sess2',
            writeonly=False,
            pbsdir=None,
            force_no_qsub=False)

        dax_tools_utils.upload_tasks(
            None,
            True,
            settings_location + 'test_upload.yaml',
            host=None,
            username=None,
            password=None,
            projects=None,
            suffix='',
            emailaddress=None)

        bin.update_tasks(
            settings_location + 'test.yaml',
            None,
            True,
            projects='proj1',
            sessions='sess1,sess2')

        dax_tools_utils.upload_tasks(
            None,
            True,
            settings_location + 'test_upload.yaml',
            host=None,
            username=None,
            password=None,
            projects=None,
            suffix='',
            emailaddress=None)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

""" setup.py

Packaging for dax
"""

from glob import glob
import os
from setuptools import setup, find_packages
import subprocess


def git_version():
    def _minimal_ext_cmd(cmd):
        env = {}
        for k in ['SYSTEMROOT', 'PATH', 'HOME']:
            v = os.environ.get(k)
            if v is not None:
                env[k] = v

        out = subprocess.check_output(cmd, stderr=subprocess.STDOUT, env=env)
        return out

    try:
        out = _minimal_ext_cmd(['git', 'rev-parse', 'HEAD'])
        GIT_REVISION = out.strip().decode('ascii')
    except OSError:
        GIT_REVISION = "Unknown"

    return GIT_REVISION


def write_git_revision_py(filename='dax/git_revision.py'):
    cnt = """
# THIS FILE IS GENERATED BY SETUP.PY
git_revision = '%(git_revision)s'
"""

    GIT_REVISION = git_version()

    with open(filename, 'w') as f:
        f.write(cnt % {'git_revision': GIT_REVISION})


def get_version():
    basedir = os.path.dirname(__file__)
    with open(os.path.join(basedir, 'dax/version.py')) as f:
        VERSION = None
        version_ns = {}
        exec(f.read(), version_ns)
        VERSION = version_ns['VERSION']
        return VERSION
    raise RuntimeError("No version found")


def readme():
    with open('README.md') as f:
        return f.read()


description = 'Distributed Automation for XNAT'

# Note: this long_description is actually a copy/paste from the top-level
# README.txt, so that it shows up nicely on PyPI.  So please remember to edit
# it only in one place and sync it correctly.
long_description = """
========================================================
DAX: Distributed Automation for XNAT
========================================================

*DAX*, is a python package developed at Vanderbilt University, Nashville, TN,
USA. It's available on github at: https://github.com/VUIIS/dax.

XNAT provides a flexible imaging informatics software platform to
organize and manage imaging data.

*DAX*, an open-source initiative under the umbrella of Vanderbilt University
Institute of Imaging Science (VUIIS), is a Python project that provides a
uniform interface to run pipelines on a cluster by grabbing data from a XNAT
database via RESTApi Calls.

*DAX* allows you to:

* extract information from XNAT via scripts (bin/Xnat_tools/Xnat...)
* create pipelines (spiders/processors) to run pipelines on your data
* build a project on XNAT with pipelines (assessors)
* launch pipelines on your cluster and upload results back to xnat
* interact with XNAT via python using commands in XnatUtils.
"""

NAME = 'dax'
MAINTAINER = 'VUIIS CCI'
MAINTAINER_EMAIL = 'vuiis-cci@googlegroups.com'
DESCRIPTION = description
LONG_DESCRIPTION = long_description
URL = "http://github.com/VUIIS/dax"
DOWNLOAD_URL = "http://github.com/VUIIS/dax"
LICENSE = 'MIT'
CLASSIFIERS = [
    # As from http://pypi.python.org/pypi?%3Aaction=list_classifiers
    "Development Status :: 5 - Production/Stable",
    "Environment :: Console",
    "Intended Audience :: Science/Research",
    "Operating System :: MacOS :: MacOS X",
    "Operating System :: POSIX",
    "Operating System :: POSIX :: Linux",
    "Operating System :: Unix",
    "Programming Language :: Python :: 3.7",
    "Topic :: Scientific/Engineering",
    "Topic :: Scientific/Engineering :: Information Analysis"]
AUTHOR = MAINTAINER
AUTHOR_EMAIL = MAINTAINER_EMAIL
PLATFORMS = ["MacOs",
             "Linux"]
VERSION = get_version()

# versions
SPHINX_MIN_VERSION = '4'
PYXNAT_MIN_VERSION = '1.1.0.2'

REQUIRES = [
    'pyxnat>=%s' % PYXNAT_MIN_VERSION,
    'pyyaml',
    'pycap',
    'pandas',
    'fpdf2',
    'PyPDF2',
    'yamale']

DOCS_REQUIRES = [
    'Sphinx>=%s' % SPHINX_MIN_VERSION]

TESTS_REQUIRES = ['nose']

BIDS_REQUIRES = ['cubids-bond-fork', 'nibabel']


if __name__ == '__main__':
    write_git_revision_py()

    setup(name=NAME,
          maintainer=MAINTAINER,
          maintainer_email=MAINTAINER_EMAIL,
          version=get_version(),
          description=DESCRIPTION,
          long_description=LONG_DESCRIPTION,
          url=URL,
          download_url=DOWNLOAD_URL,
          author=AUTHOR,
          author_email=AUTHOR_EMAIL,
          platforms=PLATFORMS,
          license=LICENSE,
          packages=find_packages(),
          include_package_data=True,
          test_suite='nose.collector',
          tests_require=TESTS_REQUIRES,
          install_requires=REQUIRES,
          python_requires='~=3.6',
          zip_safe=True,
          scripts=glob(os.path.join('bin', '*', '*')),
          classifiers=CLASSIFIERS,
          extras_require={
              'docs': DOCS_REQUIRES,
              'bids': BIDS_REQUIRES,
          })




from unittest import TestCase

import io

import yaml

from dax.processors import AutoProcessor

from dax.tests import unit_test_entity_common as common
from dax import XnatUtils
from dax import yaml_doc


class ConnectionStringUnitTest(TestCase):

    def test_a_xpath(self):
        print((XnatUtils.InterfaceTemp.A_XPATH.format(
            project='proj1', subject='subj1',
            session='sess1', assessor='assr1')))


class AutoProcessorUnitTest(TestCase):

    @staticmethod
    def _make_yaml_source(resource):
        return yaml_doc.YamlDoc().from_string(resource)

    def _construct_session(self, name):
        tpo = common.TestProjectObject(
            common.xnat_contents[name]['projects'][0]
        )
        return tpo.subjects()['subj1'].sessions()['sess1']

    def test_scan_processor_construction(self):
        yaml_source = self._make_yaml_source(
           common.processor_yamls.scan_brain_tiv_from_gif_yaml)
        ap = AutoProcessor(common.FakeXnat, yaml_source)

        yaml_source = self._make_yaml_source(common.git_pct_t1_yaml)
        ap = AutoProcessor(common.FakeXnat, yaml_source)

    def test_test(self):
        print("hello world")

    def test_get_assessor_input_types(self):
        yaml_source = self._make_yaml_source(
            common.processor_yamls.scan_brain_tiv_from_gif_yaml)
        ap = AutoProcessor(common.FakeXnat, yaml_source)
        print((ap.get_assessor_input_types()))

    # def test_scan_assessor_get_assessor_name(self):
    #     tseo = self._construct_session('brain_tiv_from_gif')
    #     tsco = tseo.scan_by_key('1')
    #
    #     yaml_source = self._make_yaml_source(
    #         common.processor_yamls.scan_brain_tiv_from_gif_yaml)
    #     ap = AutoProcessor(common.FakeXnat, yaml_source)
    #
    #     actual = ap.get_assessor_name(tsco)
    #     self.assertEquals(actual,
    #                       "proj1-x-subj1-x-sess1-x-1-x-BrainTivFromGIF_v1")

    # def test_scan_assessor_get_assessor(self):
    #     tseo = self._construct_session('brain_tiv_from_gif')
    #     tsco = tseo.scan_by_key('1')
    #
    #     yaml_source = self._make_yaml_source(
    #         common.processor_yamls.scan_brain_tiv_from_gif_yaml)
    #     ap = AutoProcessor(common.FakeXnat, yaml_source)
    #
    #     actual, name = ap.get_assessor(tsco)
    #     self.assertEquals(name,
    #                       "proj1-x-subj1-x-sess1-x-1-x-BrainTivFromGIF_v1")

    def test_scan_assessor_should_run(self):
        tseo = self._construct_session('brain_tiv_from_gif')
        tsco = tseo.scan_by_key('1')

        yaml_source = self._make_yaml_source(
            common.processor_yamls.scan_brain_tiv_from_gif_yaml)
        ap = AutoProcessor(common.FakeXnat, yaml_source)

        ret = ap.should_run(tseo.info())
        self.assertEqual(ret, 1)

    # TODO: BenM/asr_of_asr/this method needs to run off pyxnat assessor
    # objects, so create a mocked pyxnat assessor for this (and other) tests
    def test_scan_assessor_has_inputs(self):
        tseo = self._construct_session('brain_tiv_from_gif')
        tsco = tseo.scan_by_key('1')

        yaml_source = self._make_yaml_source(
            common.processor_yamls.scan_brain_tiv_from_gif_yaml)
        ap = AutoProcessor(common.FakeXnat, yaml_source)

        ret, comment = ap.has_inputs(tsco)
        self.assertEqual(ret, 1)

    def test_scan_assessor_build_cmds(self):
        tseo = self._construct_session('brain_tiv_from_gif')
        tsco = tseo.assessor_by_key('proc1')

        yaml_source = self._make_yaml_source(
            common.processor_yamls.scan_brain_tiv_from_gif_yaml)
        ap = AutoProcessor(common.FakeXnat, yaml_source)

        tsao = tseo.assessors()
        print(tsao)
        # TODO:BenM/assessor_of_assessor/we are passing an interface object
        # rather than a cached object. Fix and then re-enable
        cmds = ap.get_cmds(tsco, '/testdir')
        print(("cmds =", cmds))



import os
import stat
import netrc
from string import Template
from urllib.parse import urlparse

from .errors import DaxNetrcError

# Assessor datatypes
DEFAULT_FS_DATATYPE = 'fs:fsData'
DEFAULT_DATATYPE = 'proc:genProcData'

SLURM_JOBMEM = "sacct -j ${jobid}.batch --format MaxRss --noheader | awk '{print $1+0}'"

SLURM_JOBTIME = "sacct -j ${jobid}.batch --format CPUTime --noheader"

SLURM_COUNTJOBS = 'squeue --me --noheader | wc -l'

SLURM_COUNTJOBS_LAUNCHED = 'squeue --me --noheader | wc -l'

SLURM_COUNTJOBS_PENDING = 'squeue --me -t PENDING --noheader | wc -l'

SLURM_COUNT_PENDINGUPLOADS = 'ls -d ${resdir}/*-x-* |wc -l'

SLURM_JOBNODE = "sacct -j ${jobid}.batch --format NodeList --noheader"

SLURM_JOBSTATUS = "squeue -j ${jobid} --noheader | awk {'print $5'}"

SLURM_EXT = '.slurm'

SLURM_SUBMIT = 'sbatch'

SLURM_COMPLETE = 'slurm_load_jobs error: Invalid job id specified'

SLURM_RUNNING = 'R'

SLURM_QUEUED = 'Q'


class DAX_Netrc(object):
    """Class for DAX NETRC file containing information about XNAT logins """
    def __init__(self):
        self.netrc_file = os.path.expanduser('~/.netrc')
        if not os.path.exists(self.netrc_file):
            open(self.netrc_file, 'a').close()
            # Setting mode for the file:
            os.chmod(self.netrc_file, stat.S_IWUSR | stat.S_IRUSR)
        self.is_secured()
        self.netrc_obj = netrc.netrc(self.netrc_file)

    def is_secured(self):
        """ Check if file is secure."""
        st = os.stat(self.netrc_file)
        grp_access = bool(st.st_mode & stat.S_IRWXG)
        other_access = bool(st.st_mode & stat.S_IRWXO)
        if grp_access or other_access:
            msg = 'Wrong permissions on %s. Only user should have access.'
            raise DaxNetrcError(msg % self.netrc_file)

    def is_empty(self):
        """ Return True if no host stored."""
        return len(list(self.netrc_obj.hosts.keys())) == 0

    def has_host(self, host):
        """ Return True if host present."""
        return host in list(self.netrc_obj.hosts.keys())

    def add_host(self, host, user, pwd):
        """ Adding host to daxnetrc file."""
        netrc_template = \
"""machine {host}
login {user}
password {pwd}
"""
        parsed_host = urlparse(host).hostname

        with open(self.netrc_file, "a") as f_netrc:
            lines = netrc_template.format(host=parsed_host, user=user, pwd=pwd)
            f_netrc.writelines(lines)

    def get_hosts(self):
        """ Rerutn list of hosts from netrc file."""
        return list(self.netrc_obj.hosts.keys())

    def get_login(self, host):
        """ Getting login for a host from .netrc file."""
        parsed_host = urlparse(host).hostname

        netrc_info = self.netrc_obj.authenticators(parsed_host)
        if not netrc_info:
            msg = 'Host {} not found in netrc file.'.format(host)
            raise DaxNetrcError(msg)

        return netrc_info[0], netrc_info[2]


class DAX_Settings(object):
    def __init__(self):
        pass

    def get_user_home(self):
        return os.path.expanduser('~')

    def get_xsitype_include(self):
        return ['proc:genProcData']

    def get_cmd_submit(self):
        return SLURM_SUBMIT

    def get_prefix_jobid(self):
        return 'Submitted batch job'

    def get_suffix_jobid(self):
        return ''

    def get_cmd_count_nb_jobs(self):
        return SLURM_COUNTJOBS

    def get_cmd_count_jobs_launched(self):
        return SLURM_COUNTJOBS_LAUNCHED

    def get_cmd_count_jobs_pending(self):
        return SLURM_COUNTJOBS_PENDING

    def get_cmd_count_pendinguploads(self):
        return Template(SLURM_COUNT_PENDINGUPLOADS)

    def get_cmd_get_job_status(self):
        return Template(SLURM_JOBSTATUS)

    def get_queue_status(self):
        return SLURM_QUEUED

    def get_running_status(self):
        return SLURM_RUNNING

    def get_complete_status(self):
        return SLURM_COMPLETE

    def get_cmd_get_job_memory(self):
        return Template(SLURM_JOBMEM)

    def get_cmd_get_job_walltime(self):
        return Template(SLURM_JOBTIME)

    def get_cmd_get_job_node(self):
        return Template(SLURM_JOBNODE)

    def get_job_extension_file(self):
        return SLURM_EXT

    def get_root_job_dir(self):
        return '/tmp'

    def get_launcher_type(self):
        return 'diskq-combined'

    def get_use_reference(self):
        return False

    def get_email_opts(self):
        return 'FAIL'

    def get_job_template(self, filepath):
        filepath = os.path.expanduser(filepath)

        with open(filepath, 'r') as f:
            data = f.read()

        return Template(data)



'''
Tranform XNAT folder to BIDS format

@author: Praitayini Kanakaraj, Electrical Engineering, Vanderbilt University

'''
import os
import re
import sys
import json
import bond
import shutil
import nibabel as nib
from distutils.dir_util import copy_tree
from xml.etree import cElementTree as ET


def transform_to_bids(XNAT, DIRECTORY, project, BIDS_DIR, xnat_tag, LOGGER):
    """
    Method to move the data from XNAT folders to BIDS format (based on datatype) by looping through
    subjects/projects.
    :param XNAT: XNAT interface (connection between your python script and the XNAT database)
    :param DIRECTORY: XNAT Directory
    :param project: XNAT Project ID
    :param BIDS_DIR: BIDS Directory
    :param xnat_tag: boolean check for xnat tag
    :param LOGGER: Logging
    """
    LOGGER.info("INFO: Moving files to the BIDS folder...")
    # All the BIDS datattype
    data_type_l = ["anat", "func", "fmap", "dwi", "perf", "unknown_bids"]
    # Loop throught the XNAT folders
    for proj in os.listdir(DIRECTORY):
        if proj == project and os.path.isdir(os.path.join(DIRECTORY, proj)):
            subj_idx = 1
            for subj in os.listdir(os.path.join(DIRECTORY, proj)):
                LOGGER.info("* Subject %s" % (subj))
                sess_idx = 1
                for sess in os.listdir(os.path.join(DIRECTORY, proj, subj)):
                    LOGGER.info(" * Session %s" % (sess))
                    sess_path = os.path.join(DIRECTORY, proj, subj, sess)
                    for scan in os.listdir(sess_path):
                        if scan not in data_type_l:
                            for scan_resources in os.listdir(os.path.join(sess_path, scan)):
                                for scan_file in os.listdir(os.path.join(sess_path, scan, scan_resources)):
                                    scan_id = scan.split('-x-')[0]
                                    res_dir = (os.path.join(sess_path, scan, scan_resources))
                                    scan_path = '/projects/%s/subjects/%s/experiments/%s/scans/%s' % (
                                    project, subj, sess, scan.split('-x-')[0])
                                    scan_info = XNAT.select(scan_path)
                                    uri = XNAT.host + scan_info._uri
                                    # Do the BIDS conversion for only nifti, bval, bval scans
                                    if scan_file.endswith('.nii.gz') or scan_file.endswith(
                                            'bvec.txt') or scan_file.endswith('bval.txt') or scan_file.endswith(
                                            'bvec') or scan_file.endswith('bval') \
                                            or scan_file.lower().endswith('json'):
                                        LOGGER.info("  * Scan File %s" % (scan_file))
                                        nii_file = scan_file
                                        # Call the main BIDS function that is compatible with yaml
                                        bids_yaml(XNAT, project, scan_id, subj, res_dir, scan_file, uri, sess, nii_file,
                                                  sess_idx, subj_idx, xnat_tag)
                                        # Create the BIDS directory
                                        if not os.path.exists(os.path.join(BIDS_DIR, project)):
                                            os.makedirs(os.path.join(BIDS_DIR, project))
                                        # Move the BIDS conversion inside the XNAT resource folder to BIDS folder and then delete
                                        if os.path.exists(os.path.join(res_dir, "BIDS_DATA")):
                                            copy_tree(os.path.join(res_dir, "BIDS_DATA"),
                                                      os.path.join(BIDS_DIR, project))
                                            shutil.rmtree(os.path.join(res_dir, "BIDS_DATA"))
                                        else:
                                            # Delete XNAT files/folders after BIDS Conversion
                                            LOGGER.info("\t\t>Removing XNAT scan %s file because no BIDS datatype" % (
                                                scan_file))
                                            os.remove(os.path.join(sess_path, scan, scan_resources, scan_file))
                                if not (scan_resources.lower()=="dicom" or scan_resources.lower()=="secondary"):
                                    LOGGER.info("\t\t>Removing XNAT resource %s folder" % (scan_resources))
                                    os.rmdir(os.path.join(sess_path, scan, scan_resources))
                            LOGGER.info("\t>Removing XNAT scan %s folder" % (scan))
                            try:
                                os.rmdir(os.path.join(sess_path, scan))
                            except:
                                LOGGER.warn("\t\t>Directory is not empty")
                    sess_idx = sess_idx + 1
                    LOGGER.info("\t>Removing XNAT session %s folder" % (sess))
                    try:
                        os.rmdir(sess_path)
                    except:
                        LOGGER.warn("\t\t>Directory is not empty")
                subj_idx = subj_idx + 1
                LOGGER.info("\t>Removing XNAT subject %s folder" % (subj))
                try:
                    os.rmdir(os.path.join(DIRECTORY, proj, subj))
                except:
                    LOGGER.warn("\t\t>Directory is not empty")
    BIDS_PROJ_DIR = os.path.join(BIDS_DIR, project)
    dataset_description_file(BIDS_PROJ_DIR, XNAT, project)


def bids_yaml(XNAT, project, scan_id, subj, res_dir, scan_file, uri, sess, nii_file, sess_idx, subj_idx, xnat_tag):
    """
    Main method to put the scans in the BIDS datatype folder, create json
    sidecar and remane filenames based on the BIDS format.
    :param XNAT: XNAT interface
    :param project: XNAT project ID
    :param scan_id: Scan ID of the scan on XNAT
    :param subj: Subject of the scan on XNAT
    :param res_dir: XNAT Resource directory
    :param scan_file: Scan file
    :param uri: Link of the scan on XNAT
    :param sess: XNAT Session
    :param nii_file: Scan file (for yaml it is $col1 in the INLIST )
    :param sess_idx: Session count number
    :param subj_idx: Subject count number
    """
    # Check if the json sidecar is present or not
    if scan_file.endswith('.nii.gz'):
        is_json_present = False
        if nii_file.split('.')[0] + ".json" in os.listdir(res_dir):
            json_file = nii_file.split('.')[0] + ".json"
            is_json_present = True
        else:
            json_file = "empty.json"

    # Get the series_description and scan_type of the scan in XNAT
    project_scans = XNAT.get_project_scans(project)
    for x in project_scans:
        if x['ID'] == scan_id and x['subject_label'] == subj and x['session_label'] == sess:
            scan_type = x['type']
            series_description = x['series_description']
            scan_quality = x['scan_quality']
    # Get the xnat_type from Project level
    if XNAT.select('/data/projects/' + project + '/resources/BIDS_xnat_type/files/xnat_type.txt').exists():
        with open(XNAT.select('/data/projects/' + project + '/resources/BIDS_xnat_type/files/xnat_type.txt').get(),
                  "r+") as f:
            xnat_mapping_type = f.read()
            if xnat_mapping_type == 'scan_type':
                xnat_mapping_type = scan_type
            elif xnat_mapping_type == 'series_description':
                xnat_mapping_type = series_description
            # print(xnat_mapping_type)
    else:
        print(
            "ERROR: The type of XNAT to BIDS mapping info (for eg. scan_type or series_description) is not given. Use BidsMapping Tool")
        print("ERROR: BIDS Conversion not complete")
        sys.exit()

    # Get the datatype for the scan
    sd_dict = sd_datatype_mapping(XNAT, project)
    data_type = sd_dict.get(xnat_mapping_type, "unknown_bids")
    if data_type == "unknown_bids":
        print((
            '\t\t>WARNING: The type %s does not have a BIDS datatype mapping at default and project level. Use BidsMapping Tool' % xnat_mapping_type))

    else:
        # If datatype is known, Create the BIDS directory and do the rest
        # If XNAT_TAG is false the format is subj_<01>-ses-<01>
        sess_idx = "{0:0=2d}".format(int(sess_idx))
        subj_idx = "{0:0=2d}".format(int(subj_idx))
        bids_dir = os.path.join(res_dir, "BIDS_DATA")
        if not os.path.exists(bids_dir):
            os.makedirs(bids_dir)
        # If XNAT_TAG is true the format is subj_<SUBJID>-ses-<SESSID>
        if xnat_tag:
            sess_idx = sess
            subj_idx = subj
        data_type_dir = os.path.join(bids_dir, "sub-" + subj_idx, "ses-" + sess_idx, data_type)
        if not os.path.exists(os.path.join(bids_dir, data_type_dir)):
            os.makedirs(os.path.join(bids_dir, data_type_dir))

        # XNAT Project Scanner info
        session_info = XNAT.select('/project/' + project + '/subject/' + subj + '/experiment/' + sess).get()
        session_info = ET.fromstring(session_info)
        try:
            acquisition_site = session_info.find('{http://nrg.wustl.edu/xnat}acquisition_site').text
        except AttributeError:
            acquisition_site = 'NA'
        try:
            scanner = session_info.find('{http://nrg.wustl.edu/xnat}scanner').text
        except AttributeError:
            scanner = 'NA'
        try:
            manufacturer = session_info.find('{http://nrg.wustl.edu/xnat}scanner').attrib['manufacturer']
        except AttributeError:
            manufacturer = 'NA'
        try:
            model = session_info.find('{http://nrg.wustl.edu/xnat}scanner').attrib['model']
        except AttributeError:
            model = 'NA'

        # For only nifti scans, handle json sidecar should be checked and the json sidecar filename should changed
        if scan_file.endswith('.nii.gz'):
            xnat_prov = yaml_create_json(XNAT, data_type, res_dir, scan_file, uri, project, xnat_mapping_type, sess,
                                         is_json_present, nii_file, json_file, series_description, scan_type,
                                         scan_quality, acquisition_site, scanner, manufacturer, model)
            with open(os.path.join(res_dir, json_file), "w+") as f:
                json.dump(xnat_prov, f, indent=2)
            print('\t\t-Handling json file')
            bids_fname_json = yaml_bids_filename(XNAT, data_type, scan_id, subj, sess, project, json_file,
                                                 xnat_mapping_type, sess_idx, subj_idx, series_description)
            os.rename(os.path.join(res_dir, json_file), os.path.join(res_dir, bids_fname_json))
            shutil.move(os.path.join(res_dir, bids_fname_json), data_type_dir)

        # Change the filename and move the file
        print('\t\t-Handling nifti/bvec/bval file')
        bids_fname = yaml_bids_filename(XNAT, data_type, scan_id, subj, sess, project, scan_file, xnat_mapping_type,
                                        sess_idx, subj_idx, series_description)
        os.rename(os.path.join(res_dir, nii_file), os.path.join(res_dir, bids_fname))
        shutil.move(os.path.join(res_dir, bids_fname), data_type_dir)


def yaml_bids_filename(XNAT, data_type, scan_id, subj, sess, project, scan_file, xnat_mapping_type, sess_idx, subj_idx,
                       series_description):
    """
    Main method to put the scans in the BIDS datatype folder, create json
    sidecar and remane filenames based on the BIDS format.
    :param XNAT: XNAT interface
    :param data_type: BIDS datatype of scan
    :param scan_id: Scan id
    :param subj: XNAT Subject
    :param sess: XNAT Session
    :param project: XANT Project
    :param scan_file: Scan file on XNAT
    :param xnat_mapping_type:
    :param sess_idx: Session count number
    :param subj_idx: Subject count number
    :param series_description: series_description of the scan
    :return: BIDS filename
    """
    series_description = series_description.replace(" ","").replace('/', "").replace(":", "")
    if data_type == "anat":
        rn_dict = sd_run_mapping(XNAT, project)
        run_number = rn_dict.get(xnat_mapping_type, scan_id)
        bids_fname = "sub-" + subj_idx + '_' + "ses-" + sess_idx + '_acq-' + series_description + '_run-' + run_number + '_T1w' + \
                     '.' + ".".join(scan_file.split('.')[1:])
        return bids_fname

    elif data_type == "func":
        # Get the task for the scan
        tk_dict = sd_tasktype_mapping(XNAT, project)
        task_type = tk_dict.get(xnat_mapping_type)
        if task_type == None:
            print(('ERROR: Scan type %s does not have a BIDS tasktype mapping at default and project level ' \
                  'Use BidsMapping tool. Func folder not created' % xnat_mapping_type))
            print("ERROR: BIDS Conversion not complete")
            sys.exit()
        # Get the run_number for the scan
        rn_dict = sd_run_mapping(XNAT, project)
        # Map scan and with run_number, if run_number
        # not present for scan then 01 is used
        run_number = rn_dict.get(xnat_mapping_type, scan_id)
        bids_fname = "sub-" + subj_idx + '_' + "ses-" + sess_idx + '_task-' + task_type + '_acq-' + series_description + '_run-' \
                     + run_number + '_bold' + '.' + ".".join(scan_file.split('.')[1:])
        return bids_fname

    elif data_type == "dwi":
        rn_dict = sd_run_mapping(XNAT, project)
        run_number = rn_dict.get(xnat_mapping_type, scan_id)
        if scan_file.endswith('bvec.txt'):
            bids_fname = "sub-" + subj_idx + '_' + "ses-" + sess_idx + '_acq-' + series_description + '_run-' + run_number + '_dwi.bvec'

        elif scan_file.endswith('bval.txt'):
            bids_fname = "sub-" + subj_idx + '_' + "ses-" + sess_idx + '_acq-' + series_description + '_run-' + run_number + '_dwi.bval'
        else:
            bids_fname = "sub-" + subj_idx + '_' + "ses-" + sess_idx + '_acq-' + series_description + '_run-' + run_number + '_dwi' + \
                         '.' + ".".join(scan_file.split('.')[1:])
        return bids_fname

    elif data_type == "fmap":
        rn_dict = sd_run_mapping(XNAT, project)
        run_number = rn_dict.get(xnat_mapping_type, scan_id)
        #TODO: Include all the four different cases for fieldmap
        bids_fname = "sub-" + subj_idx + '_' + "ses-" + sess_idx + '_acq-' + series_description + '_run-' + run_number + '_fieldmap' + \
                     '.' + ".".join(scan_file.split('.')[1:])
        return bids_fname

    elif data_type == "perf":
        asl_dict = sd_asltype_mapping(XNAT, project)
        asl_type = asl_dict.get(xnat_mapping_type, scan_id)
        if asl_type == None:
            print(('ERROR: Scan type %s does not have a BIDS asltype mapping at default and project level. ' \
                  'Use BidsMapping tool. Perf folder not created' % xnat_mapping_type))
            print("ERROR: BIDS Conversion not complete")
            sys.exit()
        # Get the run_number for the scan
        rn_dict = sd_run_mapping(XNAT, project)
        run_number = rn_dict.get(xnat_mapping_type, scan_id)

        bids_fname = "sub-" + subj_idx + '_' + "ses-" + sess_idx + '_acq-' + series_description + '_run-' + run_number + '_' + asl_type + \
                '.' + ".".join(scan_file.split('.')[1:])      
        return bids_fname

def yaml_create_json(XNAT, data_type, res_dir, scan_file, uri, project, xnat_mapping_type, sess, is_json_present,
                     nii_file, json_file, series_description, scan_type, scan_quality, acquisition_site, scanner, manufacturer, model):
    """
    :param XNAT: XNAT interface
    :param data_type: BIDS datatype of the scan
    :param res_dir: XNAT Resource directory
    :param scan_file: XNAT Scan file
    :param uri: Link of scan file on XNAT
    :param project: Project ID
    :param xnat_mapping_type: Type of mapping used on XNAT (scan_type or series_description)
    :param sess: Xnat sesssion
    :param is_json_present: json sidecar present or absent in XNAT (boolean)
    :param nii_file: Scan file ($col1 in yaml)
    :param json_file: json sidecar filename
    :param series_description: series description for scan file on XNAT
    :return: xnat_prov - json that needs to be uploaded with nifti
    """
    # For dwi, fmap and anat - add only XNAT details to json sidecar
    if data_type != 'func':
        xnat_detail = {"XNATfilename": scan_file,
                       "XNATProvenance": uri,
                       "SeriesDescription": series_description,
                       "ScanType": scan_type,
                       "ScanQuality": scan_quality,
                       "AcquisitionSite": acquisition_site,
                       "ScannerManufacturer": manufacturer,
                       "Scanner": scanner,
                       "ScannerModel": model}

        if not is_json_present:
            print('\t\t>No json sidecar. Created json sidecar with xnat info.')
            xnat_prov = xnat_detail
        else:
            with open(os.path.join(res_dir, json_file), "r+") as f:
                print('\t\t>Json sidecar exists. Added xnat info.')
                xnat_prov = json.load(f)
                xnat_prov.update(xnat_detail)


    else:
        # For func check out details in nifti and json is required
        xnat_prov = yaml_func_json_sidecar(XNAT, data_type, res_dir, scan_file, uri, project, xnat_mapping_type,
                                           nii_file, is_json_present, sess, json_file, series_description, scan_type,
                                           scan_quality, acquisition_site, scanner, manufacturer, model)
    return xnat_prov


def yaml_func_json_sidecar(XNAT, data_type, res_dir, scan_file, uri, project, xnat_mapping_type, nii_file,
                           is_json_present, sess, json_file, series_description, scan_type, scan_quality, 
                           acquisition_site, scanner, manufacturer, model):
    """

    :param XNAT: XNAT interface
    :param data_type: BIDS datatype of the scan
    :param res_dir: XNAT Resource directory
    :param scan_file: XNAT Scan file
    :param uri: Link of scan file on XNAT
    :param project: Project ID
    :param xnat_mapping_type:  Type of mapping used on XNAT (scan_type or series_description)
    :param nii_file: Scan file ($col1 in yaml)
    :param is_json_present: json sidecar present or absent in XNAT (boolean)
    :param sess: XNAT Session ID
    :param json_file: json sidecar filename
    :return: json sidecare for func datatype
    """
    xnat_prov = None
    tr_dict = sd_tr_mapping(XNAT, project)
    TR_bidsmap = tr_dict.get(xnat_mapping_type)
    if TR_bidsmap == None:
        print(('\t\t>ERROR: Scan type %s does not have a TR mapping' % xnat_mapping_type))
        print("\t\t>ERROR: BIDS Conversion not complete")
        sys.exit()
    TR_bidsmap = round((float(TR_bidsmap)), 3)
    tk_dict = sd_tasktype_mapping(XNAT, project)
    task_type = tk_dict.get(xnat_mapping_type)
    img = nib.load(os.path.join(res_dir, nii_file))
    units = img.header.get_xyzt_units()[1]
    if units != 'sec':
        print("\t\t>ERROR: the units in nifti header is not secs")
        print("\t\t>ERROR: BIDS Conversion not complete")
        func_folder = os.path.dirname(bids_res_path)
        try:
            os.rmdir(func_folder)
        except:
            print('\t\t>Deletion incomplete. Directory not empty')
        sys.exit()
    TR_nifti = round(img.header['pixdim'][4].item(), 3)
    # If json not present - if TR in nifti and XNAT (project level) is equal, USE TR FROM NIFTI
    #                       if TR in nifti and XNAT (project level) is not equal, USE TR FROM XNAT (Project level) and
    #                                                                             UPDATE THE NIFTI HEADER
    if not is_json_present:
        xnat_prov = {"XNATfilename": scan_file,
                     "XNATProvenance": uri,
                     "TaskName": task_type,
                     "SeriesDescription": series_description,
                     "ScanType": scan_type,
                     "ScanQuality": scan_quality,
                     "AcquisitionSite": acquisition_site,
                     "ScannerManufacturer": manufacturer,
                     "Scanner": scanner,
                     "ScannerModel": model}

        if TR_nifti == TR_bidsmap:
            print((
                '\t\t>No existing json. TR %.3f sec in BIDS mapping and NIFTI header. Using TR %.3f sec in nifti header ' \
                'for scan file %s in session %s. ' % (TR_bidsmap, TR_bidsmap, scan_file, sess)))
            xnat_prov["RepetitionTime"] = float(TR_nifti)
        else:
            print((
                '\t\t>No existing json. WARNING: The TR is %.3f sec in project level BIDS mapping, which does not match the TR of %.3f sec in NIFTI header.\n  ' \
                '\t\tUPDATING NIFTI HEADER to match BIDS mapping TR %.3f sec for scan file %s in session %s.' \
                % (TR_bidsmap, TR_nifti, TR_bidsmap, scan_file, sess)))
            xnat_prov["RepetitionTime"] = float(TR_bidsmap)
            img.header['pixdim'][4] = TR_bidsmap
            nib.save(img, os.path.join(res_dir, nii_file))
    # If json present - if TR in JSON and XNAT (project level) is equal,     USE THE SAME JSON
    #                 - if TR in JSON and XNAT (project level) is not equal, USE TR FROM XNAT (Project level) and
    #                                                                         UPDATE THE NIFTI HEADER
    else:
        with open(os.path.join(res_dir, json_file), "r+") as f:
            xnat_prov = json.load(f)
            TR_json = round((xnat_prov['RepetitionTime']), 3)
            xnat_detail = {"XNATfilename": scan_file,
                           "XNATProvenance": uri,
                           "TaskName": task_type,
                           "SeriesDescription": series_description,
                           "ScanType": scan_type,
                           "ScanQuality": scan_quality,
                           "AcquisitionSite": acquisition_site,
                           "ScannerManufacturer": manufacturer,
                           "Scanner": scanner,
                           "ScannerModel": model}

            if TR_json != TR_bidsmap:
                print((
                    '\t\t>JSON sidecar exists. WARNING: TR is %.3f sec in project level BIDS mapping, which does not match the TR in JSON sidecar %.3f.\n ' \
                    '\t\tUPDATING JSON with TR %.3f sec in BIDS mapping and UPDATING NIFTI header for scan %s in session %s.' \
                    % (TR_bidsmap, TR_json, TR_bidsmap, scan_file, sess)))
                xnat_detail['RepetitionTime'] = float(TR_bidsmap)
                xnat_prov.update(xnat_detail)
                img.header['pixdim'][4] = TR_bidsmap
                nib.save(img, os.path.join(res_dir, nii_file))
            else:
                print((
                    '\t\t>JSON sidecar exists. TR is %.3f sec in BIDS mapping and JSON sidecar for scan %s in session %s. ' \
                    'Created json sidecar with XNAT info' \
                    % (TR_bidsmap, scan_file, sess)))
                xnat_prov.update(xnat_detail)
    return xnat_prov


def sd_tr_mapping(XNAT, project):
    """
    Method to get the Repetition Time mapping from Project level
    :param XNAT: XNAT interface
    :param project: XNAT Project ID
    :return: Dictonary with scan_type/series_description and  Repetition Time mapping
    """
    tr_dict = {}
    if XNAT.select('/data/projects/' + project + '/resources/BIDS_repetition_time_sec').exists():
        for res in XNAT.select('/data/projects/' + project + '/resources/BIDS_repetition_time_sec/files').get():
            if res.endswith('.json'):
                with open(XNAT.select(
                        '/data/projects/' + project + '/resources/BIDS_repetition_time_sec/files/' + res).get(),
                          "r+") as f:
                    tr_mapping = json.load(f)
                    print(('\t\t>Using BIDS Repetition Time in secs mapping in project level %s' % (project)))
                    tr_dict = tr_mapping[project]

    else:
        print("\t\t>ERROR: no TR mapping at project level. Func folder not created")
        print("\t\t>ERROR: BIDS Conversion not complete")
        sys.exit()
    return tr_dict

def sd_asltype_mapping(XNAT, project):
    """
    Method to get the asl type mapping from Project level
    Mapping options: asl, m0scan

    :param XNAT: XNAT interface
    :param project: XNAT Project ID
    :return: Dictonary with scan_type/series_description and  asl type mapping
    """
    asl_dict = {}
    if XNAT.select('/data/projects/' + project + '/resources/BIDS_asltype').exists():
        for res in XNAT.select('/data/projects/' + project + '/resources/BIDS_asltype/files').get():
            if res.endswith('.json'):
                with open(XNAT.select(
                        '/data/projects/' + project + '/resources/BIDS_asltype/files/' + res).get(),
                          "r+") as f:
                    asl_mapping = json.load(f)
                    print(('\t\t>Using BIDS asltype mapping in project level %s' % (project)))
                    asl_dict = asl_mapping[project]

    else:
        print("\t\t>ERROR: no asl mapping at project level. Perf folder not created")
        print("\t\t>ERROR: BIDS Conversion not complete")
        sys.exit()
    return asl_dict

def sd_datatype_mapping(XNAT, project):
    """
    Method to get the Datatype mapping from Project level
    :param XNAT: XNAT interface
    :param project: XNAT Project ID
    :return: Dictonary with scan_type/series_description and datatype mapping
    """
    sd_dict = {}

    if XNAT.select('/data/projects/' + project + '/resources/BIDS_datatype').exists():
        for res in XNAT.select('/data/projects/' + project + '/resources/BIDS_datatype/files').get():
            if res.endswith('.json'):
                with open(XNAT.select('/data/projects/' + project + '/resources/BIDS_datatype/files/' + res).get(),
                          "r+") as f:
                    datatype_mapping = json.load(f)
                    sd_dict = datatype_mapping[project]
                    print(('\t\t>Using BIDS datatype mapping in project level %s' % (project)))

    else:
        # Edit here to set default mapping options
        print(('\t\t>WARNING: No BIDS datatype mapping in project %s - using default mapping' % (project)))
        scans_list_global = XNAT.get_project_scans('LANDMAN')

        for sd in scans_list_global:
            c = re.search('T1|T2|T1W', sd['scan_type'])
            if not c == None:
                sd_anat = sd['scan_type'].strip().replace('/', '_').replace(" ", "").replace(":", '_')
                sd_dict[sd_anat] = "anat"

        for sd in scans_list_global:
            c = re.search('rest|Resting state|Rest', sd['scan_type'], flags=re.IGNORECASE)
            if not c == None:
                sd_func = sd['scan_type'].strip().replace('/', '_').replace(" ", "").replace(":", '_')
                sd_dict[sd_func] = "func"

        for sd in scans_list_global:
            c = re.search('dwi|dti', sd['scan_type'], flags=re.IGNORECASE)
            if not c == None:
                sd_dwi = sd['scan_type'].strip().replace('/', '_').replace(" ", "").replace(":", '_')
                sd_dict[sd_dwi] = "dwi"

        for sd in scans_list_global:
            c = re.search('Field|B0', sd['scan_type'], flags=re.IGNORECASE)
            if not c == None:
                sd_fmap = sd['scan_type'].strip().replace('/', '_').replace(" ", "").replace(":", '_')
                sd_dict[sd_fmap] = "fmap"
        with open("global_mapping.json", "w+") as f:
            json.dump(sd_dict, f, indent=2)

    return sd_dict


def sd_tasktype_mapping(XNAT, project):
    """
    Method to get the Task type mapping at Project level
    :param XNAT: XNAT interface
    :param project: XNAT Project ID
    :return: Dictonary with scan_type/series_description and tasktype mapping
    """
    tk_dict = {}
    if XNAT.select('/data/projects/' + project + '/resources/BIDS_tasktype').exists():
        for res in XNAT.select('/data/projects/' + project + '/resources/BIDS_tasktype/files').get():
            if res.endswith('.json'):
                with open(XNAT.select('/data/projects/' + project + '/resources/BIDS_tasktype/files/'
                                      + res).get(), "r+") as f:
                    datatype_mapping = json.load(f)
                    tk_dict = datatype_mapping[project]

    else:
        print(('\t\t>WARNING: No BIDS task type mapping in project %s - using default mapping' % (project)))
        scans_list_global = XNAT.get_project_scans('LANDMAN')
        for sd in scans_list_global:
            c = re.search('rest|Resting state|Rest', sd['scan_type'], flags=re.IGNORECASE)
            if not c == None:
                sd_func = sd['scan_type'].strip().replace('/', '_').replace(" ", "").replace(":", '_')
                tk_dict[sd_func] = "rest"
        with open("global_tk_mapping.json", "w+") as f:
            json.dump(tk_dict, f, indent=2)
    return tk_dict


def sd_run_mapping(XNAT, project):
    """
    Method to get the Run number mapping at Project level
    :param XNAT: XNAT interface
    :param project: XNAT Project ID
    :return: Dictonary with scan_type/series_description and run number mapping
    """
    rn_dict = {}
    if XNAT.select('/data/projects/' + project + '/resources/BIDS_run_number').exists():
        for res in XNAT.select('/data/projects/' + project + '/resources/BIDS_run_number/files').get():
            if res.endswith('.json'):
                with open(XNAT.select('/data/projects/' + project + '/resources/BIDS_run_number/files/'
                                      + res).get(), "r+") as f:
                    datatype_mapping = json.load(f)
                    rn_dict = datatype_mapping[project]

    else:
        print("\t\t>WARNING: No Run number mapping at project level. Using scan id as run number")

    return rn_dict

def dataset_description_file(BIDS_PROJ_DIR, XNAT, project):
    """
    Build BIDS dataset description json file
    :param BIDS_PROJ_DIR: Project BIDS directory
    :param XNAT: XNAT interface
    :param project: XNAT Project
    """
    BIDSVERSION = "1.0.1"
    dataset_description = dict()
    dataset_description['BIDSVersion'] = BIDSVERSION
    dataset_description['Name'] = project
    dataset_description['DatasetDOI'] = XNAT.host
    project_info = XNAT.select('/project/' + project).get()
    project_info = ET.fromstring(project_info)
    PI_element = project_info.findall('{http://nrg.wustl.edu/xnat}PI')
    if len(PI_element) > 0:
        dataset_description['Author'] = PI_element[0][1].text, PI_element[0][0].text
    else:
        dataset_description['Author'] = "No Author defined on XNAT"
    if not os.path.exists(BIDS_PROJ_DIR):
        os.makedirs(BIDS_PROJ_DIR)
    with open(os.path.join(BIDS_PROJ_DIR, 'dataset_description.json'), 'w+') as f:
        json.dump(dataset_description, f, indent=2)

class XNATBond(object):
    def __init__(self, bids_dir):
        self.bids_dir = bids_dir

    def generate_params(self, bond_dir):
        if not os.path.exists(bond_dir):
            os.makedirs(bond_dir)
        bond.BOnD(self.bids_dir).get_CSVs(os.path.join(bond_dir, 'keyparam_original'))

    def edit_params(self, keyparam_edited,keyparam_files,new_keyparam_prefix):
        print(self.bids_dir,keyparam_edited,keyparam_files,new_keyparam_prefix)
        bond.BOnD(self.bids_dir).apply_csv_changes(keyparam_edited,keyparam_files,new_keyparam_prefix)